#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2006 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////


#include "JetByteTools/CoreTools/tstring.h"

#include "JetByteTools/OpenSSLTools/StreamSocketServer.h"
#include "JetByteTools/OpenSSLTools/StreamSocketServerExCallback.h"

///////////////////////////////////////////////////////////////////////////////
// CSocketServer
///////////////////////////////////////////////////////////////////////////////

class CSocketServer :
   public JetByteTools::OpenSSL::CStreamSocketServerEx,
   private JetByteTools::OpenSSL::CStreamSocketServerExCallback
{
   public :

      CSocketServer(
         const JetByteTools::OpenSSL::CContext &sslContext,
         bool verifyPeer,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::SocketBufferSize recvBufferSize,
         JetByteTools::Socket::SocketBufferSize sendBufferSize,
         JetByteTools::Socket::ZeroByteReadConfiguration zeroByteReadConfiguration,
         const JetByteTools::OpenSSL::CApplicationProtocolSelection &protocolSelection);

      CSocketServer(
         const JetByteTools::OpenSSL::CContext &sslContext,
         bool verifyPeer,
         const JetByteTools::Socket::IFullAddress &address,
         JetByteTools::Socket::ListenBacklog listenBacklog,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::Socket::IAllocateSequencedStreamSockets &socketAllocator,
         JetByteTools::IO::IAllocateBuffers &bufferAllocator,
         JetByteTools::Socket::SocketBufferSize recvBufferSize,
         JetByteTools::Socket::SocketBufferSize sendBufferSize,
         JetByteTools::Socket::ZeroByteReadConfiguration zeroByteReadConfiguration);

      CSocketServer(
         const CSocketServer &rhs) = delete;

      ~CSocketServer();

      CSocketServer &operator=(
         const CSocketServer &rhs) = delete;

   private :

      typedef JetByteTools::OpenSSL::CStreamSocketServerEx Base;

      // Implement just the bits of IStreamSocketServerCallback that we need

      void OnSecureConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnConnectionEstablished(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Socket::IAddress &address) override;

      void OnSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnSecureSocketReleased(
         JetByteTools::Core::IIndexedOpaqueUserData &userData) override;

      void OnError(
         const JetByteTools::Core::_tstring &message) override;

      void OnReadCompleted(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) override;

      int Verify(
         int preverify_ok,
         X509_STORE_CTX *pContext) override;

      const JetByteTools::OpenSSL::CApplicationProtocolSelection *OnApplicationLayerProtocolNegotiation(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::OpenSSL::CApplicationProtocolSelection &offeredProtocols) override;

      void OnApplicationLayerProtocolNegotiated(
         JetByteTools::Socket::IStreamSocket &socket,
         const JetByteTools::Core::_tstring &protocol) override;

      void EchoMessage(
         JetByteTools::Socket::IStreamSocket &socket,
         JetByteTools::IO::CSmartBuffer &buffer) const;

      const JetByteTools::OpenSSL::CApplicationProtocolSelection *m_pProtocolSelection;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: SocketServer.h
///////////////////////////////////////////////////////////////////////////////
