#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: TestMessage.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2005 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "ITestMessage.h"

#include "JetByteTools/CoreTools/tstring.h"
#include "JetByteTools/CoreTools/PerformanceCounter.h"

#include <vector>

///////////////////////////////////////////////////////////////////////////////
// CTestMessage
///////////////////////////////////////////////////////////////////////////////

class CTestMessage : public ITestMessage
{
   public :

      explicit CTestMessage(
         DataLength messageSize);

      CTestMessage(
         const CTestMessage &rhs) = delete;

      CTestMessage &operator=(
         const CTestMessage &rhs) = delete;

      // Implement ITestMessage

      virtual ~CTestMessage();

      void ExpectRepeats(
         size_t numberOfRepeats) override;

      JetByteTools::Core::_tstring GetIdentifier() const override;

      const Times &GetTimes() const override;

      const BYTE *GetBytes() const override;

      DataLength GetSize() const override;

      bool ValidateResponse(
         const BYTE *pData,
         DataLength dataLength,
         DataLength &dataUsed) const override;

      DataLength GetRecvOffset() const override;

      DataLength GetResponseSize() const override;

      bool BytesSent(
         DataLength bytesSent) override;

      void ResetMessage() override;

   private :

      const DataLength m_messageSize;

      BYTE *m_pData;

      mutable DataLength m_sendOffset;

      mutable DataLength m_receiveOffset;

      mutable Times m_times;

      mutable size_t m_sendIndex;

      size_t m_recvIndex;

      JetByteTools::Core::CPerformanceCounter m_counter;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: TestMessage.h
///////////////////////////////////////////////////////////////////////////////
