#pragma once
///////////////////////////////////////////////////////////////////////////////
// File: AsyncFileLog.h
///////////////////////////////////////////////////////////////////////////////
//
// Copyright 2019 JetByte Limited.
//
// This software is provided "as is" without a warranty of any kind. All
// express or implied conditions, representations and warranties, including
// any implied warranty of merchantability, fitness for a particular purpose
// or non-infringement, are hereby excluded. JetByte Limited and its licensors
// shall not be liable for any damages suffered by licensee as a result of
// using the software. In no event will JetByte Limited be liable for any
// lost revenue, profit or data, or for direct, indirect, special,
// consequential, incidental or punitive damages, however caused and regardless
// of the theory of liability, arising out of the use of or inability to use
// software, even if JetByte Limited has been advised of the possibility of
// such damages.
//
///////////////////////////////////////////////////////////////////////////////

#include "JetByteTools/IOTools/AsyncFileLog.h"

///////////////////////////////////////////////////////////////////////////////
// Classes defined in other files...
///////////////////////////////////////////////////////////////////////////////

namespace JetByteTools
{
   namespace Win32
   {
      class CManualResetEvent;
   }
}

class CSimpleServerShutdownHandler;

///////////////////////////////////////////////////////////////////////////////
// CAsyncFileLog
///////////////////////////////////////////////////////////////////////////////

class CAsyncFileLog : public JetByteTools::IO::CAsyncFileLog
{
   public :

      explicit CAsyncFileLog(
         const JetByteTools::Core::_tstring &filename,
         JetByteTools::IO::ILimitPendingWrites &writeLimiter = JetByteTools::IO::CPendingWriteLimiter::NoLimitLimiter);

      CAsyncFileLog(
         const JetByteTools::Core::_tstring &filename,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::IO::IAllocateBuffers &allocator,
         JetByteTools::IO::ILimitPendingWrites &writeLimiter = JetByteTools::IO::CPendingWriteLimiter::NoLimitLimiter);

      CAsyncFileLog(
         const JetByteTools::Core::_tstring &filename,
         JetByteTools::IO::IIOPool &pool,
         JetByteTools::IO::IAllocateBuffers &allocator,
         const JetByteTools::Core::IProvideLocalTime &timeProvider,
         JetByteTools::IO::ILimitPendingWrites &writeLimiter = JetByteTools::IO::CPendingWriteLimiter::NoLimitLimiter);

      CAsyncFileLog(
         const CAsyncFileLog &rhs) = delete;

      CAsyncFileLog &operator=(
         const CAsyncFileLog &rhs) = delete;

      void SetShutdownHandler(
         CSimpleServerShutdownHandler &shutdownHandler);

      void SetShutdownEvent(
         JetByteTools::Core::CManualResetEvent &event);

   private :

      // Implement JetByteTools::IO::CAsyncFileWriter::Callback

      void OnError(
         JetByteTools::IO::CAsyncFileWriter *pInstance,
         const JetByteTools::Core::_tstring &where,
         DWORD lastError) override;

      void OnError(
         JetByteTools::IO::CAsyncFileWriter *pInstance,
         const JetByteTools::Core::_tstring &where,
         const JetByteTools::Core::_tstring &message) override;

      void OnLastWrite(
         JetByteTools::IO::CAsyncFileWriter *pInstance) override;

      void OnPendingWriteLimit(
         JetByteTools::IO::CAsyncFileWriter *pInstance) override;

      // Implement IMonitorIOPool

      void OnThreadPoolThreadDestroyed() override;

      void OnThreadPoolError(
         const JetByteTools::Core::_tstring &message) override;

      CSimpleServerShutdownHandler *m_pShutdownHandler;

      JetByteTools::Core::CManualResetEvent *m_pEvent;
};

///////////////////////////////////////////////////////////////////////////////
// End of file: AsyncFileLog.h
///////////////////////////////////////////////////////////////////////////////
